
local AddonData = Inspect.Addon.Detail("CarnageBook")
CB = AddonData.data

CB.Carnages = {}
CB.Lang = Inspect.System.Language()

CB.OutputConsole = "general"

CB.ZoneIDs = {
	z698CB7B72B3D69E9 = "Cape Jule",
	z754553DD46F46371 = "City Core",
	z48530386ED2EA5AD = "Eastern Holdings",
	z563CB77E4A32233F = "Ardent Domain",
	z4D8820D7EF52685C = "Kingsward",
	z2F1E4708BEC6A608 = "Ashora",
	z1C938C07F41C83CC = "Pelladane",
	z59124F7DD7F15825 = "Seratos",
	z39095BA75AD7DC03 = "Morban",
	z2F9C9E1FF91F9293 = "Steppes of Infinity",
	z10D7E74AB6D7B293 = "The Dendrome",
}

CB.ZoneID_ZoneName = {}
CB.ZoneName_ZoneID = {}

if CB.Lang ~= "English" and CB.Lang ~= "French" and CB.Lang ~= "German" then
	CB.Lang = "English"
end

local function PrintLine(s)
	Command.Console.Display(CB.OutputConsole, false, s, false)
end

local function SlashHandler(h, args)
	CB.OutputConsole = "general"
	for k, _ in pairs(Inspect.Console.List()) do
		local c = Inspect.Console.Detail(k)
		if c and c.name == "Carnage Book" then
			CB.OutputConsole = k
		end
	end
		
	if string.len(args) == 0 then
		local player = Inspect.Unit.Detail('player')
		if player and player.zone and CB.ZoneID_ZoneName[player.zone] then
			args = CB.ZoneID_ZoneName[player.zone]
		end
		if string.len(args) == 0 then
			PrintLine("No carnages found for current zone.")
			return
		end
	end

	local print_completed = false
	local print_started = false
	local print_missing = true

	local complete = {}
	local current = {}

	local missedtotal = 0
	local currenttotal = 0
	local completedtotal = 0

	local idlen = nil
	local shortid = nil
	local longid = nil

	local zid = CB.ZoneName_ZoneID[args]

	if not zid then
		zid = CB.ZoneName_ZoneID[string.lower(args)]
	end

	if zid and CB.Carnages[CB.ZoneIDs[zid]] then
		for id, _ in pairs(Inspect.Quest.Complete()) do
			idlen = string.len(id)
			shortid = string.format("q%sxxxxxxxx", string.sub(id, idlen-15, idlen-8))
			complete[id] = true
			complete[shortid] = true
		end

		for id, _ in pairs(Inspect.Quest.List()) do
			idlen = string.len(id)
			shortid = string.format("q%sxxxxxxxx", string.sub(id, idlen-15, idlen-8))
			current[id] = true
			current[shortid] = true
		end

		PrintLine(string.format("Printing out missing carnages for '%s'", args))

		for id, quest in pairs(CB.Carnages[CB.ZoneIDs[zid]]) do
			shortid = string.format("q%sxxxxxxxx", string.sub(id, 2, 9))
			longid = string.format("qFFFFFFFF%sxxxxxxxx", string.sub(id, 2, 9))
			if complete[id] or complete[shortid] or complete[longid] then
				if print_completed then
					PrintLine(string.format("Completed - %s", quest[CB.Lang].Name))
				end
				completedtotal = completedtotal + 1
			elseif current[id] or current[shortid] or current[longid] then
				if print_started then
					PrintLine(string.format("Started      - %s", quest[CB.Lang].Name))
				end
				currenttotal = currenttotal + 1
			else
				if print_missing then
					PrintLine(string.format("  %s (%s)", quest[CB.Lang].Name, quest[CB.Lang].Summary))
				end
				missedtotal = missedtotal + 1
			end
		end
		PrintLine(string.format("Total: %d, Completed: %d, Current: %d, Missing: %d", (completedtotal+currenttotal+missedtotal), completedtotal, currenttotal, missedtotal))
	else
		PrintLine(string.format("No zone found (%s)", args))
	end
end

local function InitZones(h, ModID)
	if ModID == "CarnageBook" then
		for z, name in pairs(CB.ZoneIDs) do
			local ztmp = Inspect.Zone.Detail(z)
			if ztmp and ztmp.name then
				CB.ZoneID_ZoneName[z] = ztmp.name
				CB.ZoneName_ZoneID[ztmp.name] = z
				CB.ZoneName_ZoneID[string.lower(ztmp.name)] = z
			else
				CB.ZoneID_ZoneName[z] = name
			end
			CB.ZoneName_ZoneID[name] = z
			CB.ZoneName_ZoneID[string.lower(name)] = z
		end
		Command.Console.Display("general", false, "<font color=\"#FFFF00\">Carnage Book "..AddonData.toc.Version.." Loaded.</font>", true)
		Command.Console.Display("general", false, "<font color=\"#00FFFF\">/carnage</font>   Will list out missing carnage quests based on your current zone.", true)
		Command.Console.Display("general", false, "<font color=\"#00FFFF\">/carnage Zone Name</font>   Will list out missing carnage quests for the given zone.", true)
		Command.Console.Display("general", false, "Output will appear in a chat window named \"Carnage Book\" if it exists, otherwise General", false)
	end
end

Command.Event.Attach(Event.Addon.Load.End, InitZones, "Init zone tables")
Command.Event.Attach(Command.Slash.Register("carnage"), SlashHandler, "Slash Command")
