local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local AdvancedRaidFramesWidth = 140
local AdvancedRaidFramesTopBarHeight = 30
local AdvancedRaidFramesBottomBarHeight = 16
local AdvancedRaidFramesMargin = 2
local AdvancedRaidFramesHeight = AdvancedRaidFramesTopBarHeight + AdvancedRaidFramesBottomBarHeight 
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local AdvancedRaidFrames = WT.UnitFrame:Template("AdvancedRaidFrames")
AdvancedRaidFrames.Configuration.Name = "Advanced Raid Frames"
AdvancedRaidFrames.Configuration.RaidSuitable = true
AdvancedRaidFrames.Configuration.FrameType = "Frame"
AdvancedRaidFrames.Configuration.Width = AdvancedRaidFramesWidth + 2
AdvancedRaidFrames.Configuration.Height = AdvancedRaidFramesHeight + 2
AdvancedRaidFrames.Configuration.Resizable = { AdvancedRaidFramesWidth + 2, AdvancedRaidFramesHeight + 2, 300, 100 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function AdvancedRaidFrames:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------
local function CalculateCastChanges()
	if WT.Units then
		for unitChange in pairs(WT.Units) do
			local unitstmp = Inspect.Unit.Detail(unitChange)
			local metmp = Inspect.Unit.Detail("player")
			--dump(Inspect.Unit.Detail(unitChange).role)
			WT.Units[unitChange].selectColor = { r = 0, g = 0, b = 0, a = 1 }
			if WT.Units[unitChange].id == Inspect.Unit.Lookup("player.target") then
				WT.Units[unitChange].selectColor = { r = 1, g = 1, b = 1, a = 1 }
			end
			if unitstmp then 
				if unitstmp.absorb ~= nil then
					WT.Units[unitChange].Absorb = (unitstmp.absorb / unitstmp.healthMax) * 100
					WT.Units[unitChange].AbsorbColor = { r=0,g=1,b=1,a=0.8}
				else
					WT.Units[unitChange].Absorb = nil
				end
				if unitstmp.availability == 'partial' then
					WT.Units[unitChange].CrachColor = { r = 0, g = 1, b = 1, a = 1 }
				end
				if unitstmp.availability == 'none' then
					WT.Units[unitChange].CrachColor = { r = 0, g = 0, b = 1, a = 1 }
				end
				if unitstmp.availability == 'full' then
					WT.Units[unitChange].CrachColor = { r = 0, g = 0, b = 0, a = 0 }
				end
				if WT.Units[unitChange].cleansable then
					WT.Units[unitChange].cleansablecolor = { r = 0, g = 0, b = 0.9, a = 1 }
				else 
					WT.Units[unitChange].cleansablecolor = WT.Units[unitChange].callingColor
				end
				if unitstmp.coordX and unitstmp.coordY and unitstmp.coordZ and metmp.coordX and metmp.coordY and metmp.coordZ then
					local distance = WT.Utility.MeasureDistanceSquared(unitstmp.coordX, unitstmp.coordY, unitstmp.coordZ,metmp.coordX, metmp.coordY, metmp.coordZ) 
					if distance >= 1296 then
						WT.Units[unitChange].blockeddistance = true
						WT.Units[unitChange].blocked = false -- disable blocked makes them to dark
					else 
						WT.Units[unitChange].blockeddistance = false
					end
				end
				
			end 
		end
	end
end
local function OnSystemUpdateBegin()
	CalculateCastChanges()
end
WT.Unit.CreateVirtualProperty("raidStatus1", { "offline", "afk", "dead" },
	function(unit)
		if unit.offline then
			return "(OFF) "
		elseif unit.afk then
			return "(AFK) "
		elseif unit.dead then
			return ""
		else
			return ""
		end
	end)
table.insert(Event.System.Update.Begin,			{ OnSystemUpdateBegin, AddonId, AddonId .. "_DB_OnSystemUpdateBegin" })
function AdvancedRaidFrames:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				}, 				
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.8,
				color={r=0,g=0,b=0,a=0.6}, colorBinding="aggroColor",
			},
			{
				-- Generic Element Configuration
				id="frameBackdrop3", type="Frame", parent="frameBackdrop", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				}, 				
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.8,
				color={r=0,g=0,b=0,a=0.6}, colorBinding="CrachColor",
			},
			{
				-- Generic Element Configuration
				id="frameBlockedDistance", type="Frame", parent="frameBackdrop", layer=15, visibilityBinding="blockeddistance",
				color={r=0,g=0,b=0},alpha=0.8,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, visibilityBinding="blocked",
				color={r=0,g=0,b=0},alpha=0.8,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="barResource", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=-3 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-3 },
				},
				-- visibilityBinding="id",
				-- Type Specific Element Configuration
				binding="resourcePercent", height=AdvancedRaidFramesBottomBarHeight, colorBinding="resourceColor",
				texAddon=AddonId,texFile="img/Healbot.png",
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="barResource", layer=10,
				attach = {
					{ point="CENTERLEFT", element="frame", targetPoint="CENTERLEFT", offsetX=3, offsetY=0 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-3 },
				},
				-- visibilityBinding="id",
				-- Type Specific Element Configuration
				growthDirection="right",
				binding="healthPercent", height=AdvancedRaidFramesBottomBarHeight, colorBinding="cleansablecolor",
				texAddon=AddonId, texFile="img/Healbot.png",
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				-- Generic Element Configuration
				id="barAbsorb", type="Bar", parent="barResource", layer=11,
				attach = {
					{ point="CENTERLEFT", element="frame", targetPoint="CENTERLEFT", offsetX=3, offsetY=0 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-3 },
				},
				-- visibilityBinding="id",
				-- Type Specific Element Configuration
				growthDirection="right",
				binding="Absorb", height=AdvancedRaidFramesBottomBarHeight, colorBinding="AbsorbColor",
				media="wtBantoBar",
				texAddon=AddonId,texFile="img/Healbot.png",
			},
			{
				-- Generic Element Configuration
				id="labelHealthR", type="Label", parent="barAbsorb", layer=20,
				attach = {{ point="CENTERRIGHT", element="barAbsorb", targetPoint="CENTERRIGHT", offsetX=0, offsetY=0 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}/{healthMax} ({healthPercent}%)", default=" ", fontSize=12
			},
			{
				id="imgRole", type="ImageSet", parent="frameBackdrop", layer=20,
				attach = {{ point="CENTERLEFT", element="frameBackdrop", targetPoint="TOPLEFT", offsetX=1, offsetY=10 }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/Roles12.png", nameBinding="role", cols=1, rows=5, 
				names = { ["tank"] = 0, ["heal"] = 1, ["dps"] = 2, ["support"] = 3 }, defaultIndex = "hide"
			},

			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frameBackdrop", layer=20,
				attach = {{ point="CENTERLEFT", element="imgRole", targetPoint="CENTERRIGHT", offsetX=0, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameShort}", default=" ", fontSize=12, 
			},
			{
				-- Generic Element Configuration
				id="labelresource", type="Label", parent="barResource", layer=20,
				attach = {{ point="CENTERLEFT", element="barResource", targetPoint="CENTERLEFT", offsetX=0, offsetY=2 }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="{resource}/{resourceMax} ({resourcePercent}%)", default=" ", fontSize=10
			},
			{
				-- Generic Element Configuration
				id="labelresource1", type="Label", parent="frameBackdrop", layer=20,
				attach = {{ point="CENTERRIGHT", element="frameBackdrop", targetPoint="TOPRIGHT", offsetX=0, offsetY=10 }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="{planar}/{planarMax}", default=" ", fontSize=10
			},
			{
				-- Generic Element Configuration
				id="labelPVP", type="Label", parent="frameBackdrop", layer=20,
				attach = {{ point="CENTERLEFT", element="labelName", targetPoint="CENTERRIGHT", offsetX=-5, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="({pvpflag})", default=" ", fontSize=12
			},
			{
				id="imgReady", type="ImageSet", parent="frameBackdrop", layer=30,
				attach = {{ point="CENTER", element="frame", targetPoint="CENTER" }}, -- visibilityBinding="id",
				texAddon=AddonId, texFile="img/wtReady.png", nameBinding="readyStatus", cols=1, rows=2, 
				names = { ["ready"] = 0, ["notready"] = 1 }, defaultIndex = "hide"
			},
			{
				-- Generic Element Configuration
				id="labelStatus", type="Label", parent="labelPVP", layer=20,
				attach = {{ point="CENTERLEFT", element="labelPVP", targetPoint="CENTERRIGHT", offsetX=-5, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{raidStatus1}", default=" ", fontSize=12
			},
			{
				-- Generic Element Configuration
				id="labelMark", type="MediaSet", parent="labelStatus", layer=20,
				attach = {{ point="CENTERLEFT", element="labelStatus", targetPoint="CENTERRIGHT", offsetX=-5, offsetY=0 }},
			    width = 12, height = 12,
			    nameBinding="mark",
			    names = 
			    {
			        ["1"] = "riftMark01_mini",
			        ["2"] = "riftMark02_mini",
			        ["3"] = "riftMark03_mini",
			        ["4"] = "riftMark04_mini",
			        ["5"] = "riftMark05_mini",
			        ["6"] = "riftMark06_mini",
			        ["7"] = "riftMark07_mini",
			        ["8"] = "riftMark08_mini",
			        ["9"] = "riftMark09_mini",
			        ["10"] = "riftMark10_mini",
			        ["11"] = "riftMark11_mini",
			        ["12"] = "riftMark12_mini",
			        ["13"] = "riftMark13_mini",
			        ["14"] = "riftMark14_mini",
			        ["15"] = "riftMark15_mini",
			        ["16"] = "riftMark16_mini",
			        ["17"] = "riftMark17_mini",
			    },
			    visibilityBinding="mark",alpha=1.0,
			},
			{
				-- Generic Element Configuration
				id="frameBackdrop2", type="Frame", parent="frameBackdrop", layer=1,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				}, 				
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.2,
				color={r=0,g=0,b=0,a=0}, colorBinding="selectColor",
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frameBackdrop", layer=30,
				attach = {{ point="BOTTOMRIGHT", element="frameBackdrop", targetPoint="BOTTOMRIGHT", offsetX=-2, offsetY=-2 }},
				rows=2, cols=4, iconSize=16, iconSpacingHorizontal=1, iconSpacingVertical=1, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up",
				sweepOverlay=false,
			},
		}
	}
	
	for idx,element in ipairs(template.elements) do
		if options.excludeBuffs and element.type=="BuffPanel" then
			-- Don't add buff panels if excluding them
		else
			self:CreateElement(element)
		end 
	end
	
	if options.showBackground then
		self:SetBackgroundColor(0,0,0,0.2)
	end
	
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end