local addon, data = ...

_RESOURCETRACKER = {}
local RT = _RESOURCETRACKER

local default_settings = {
	mmx=200, mmy=200, locked = false, tracked = {}, fontsize = 36, num_nodes = 5, fade_time = 2,
	cfgx = 500, cfgy = 200, trx = -1, try = -1, showcombat = true, version=addon.toc.Version
	}

local IDX_WOOD = 1
local IDX_ORE = 2
local IDX_PLANTS = 3
local IDX_FISH = 4

local default_settings_node = {
	[IDX_WOOD] = {}, [IDX_ORE] = {}, [IDX_PLANTS] = {}, [IDX_FISH] = {}, version=addon.toc.version
}

local SVLOADED = false
local ITEMS = data.ITEMS

local LNG = data.SYSLANG

RT.ALLITEMS = {}
RT.LOOKUP = data.LOOKUP

local ICONS = {}
local RT_ACTIVE = true

MESSAGE_STACK = {
}

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function RT.ShowMessage(m)
	local tx = Inspect.Time.Real()
	local rq = true
	for k,v in pairs(MESSAGE_STACK) do
		if v.i == m.id then
			MESSAGE_STACK[k].t = nil
			rq = false
			break
		end
	end
	if rq then
		table.insert(MESSAGE_STACK, 1, {i=m.id, t=nil, x=m.coordX, y=m.coordY, z=m.coordZ, n=m.description, d=1000})
	end
	if #MESSAGE_STACK > 10 then
		table.remove(MESSAGE_STACK, 11)
	end
end

function RT.Event_Map_Remove(h,e)
	for ed, ev in pairs(e) do
		for k,v in pairs(MESSAGE_STACK) do
			if v.i == ed then
				if v.d <= 70 then
					MESSAGE_STACK[k].t = Inspect.Time.Real()-100
				else
					MESSAGE_STACK[k].t = Inspect.Time.Real()+ResourceTracker_Settings.fade_time
				end
				break
			end
		end
	end
end

function RT.Event_Map_Add(h,e)
	for k,v in pairs(Inspect.Map.Detail(e)) do
		if ResourceTracker_Settings.tracked[v.description] and RT_ACTIVE then
			RT.ShowMessage(v)
		end
		local res = RT.ALLITEMS[v.description]
		if res then
			local ix = string.format("%d.%d.%d", v.coordX, v.coordZ, v.coordY)
			if ResourceTracker_Nodes[res][LIBZONECHANGE.currentZoneID] == nil then
				ResourceTracker_Nodes[res][LIBZONECHANGE.currentZoneID] = {}
			end
			if ResourceTracker_Nodes[res][LIBZONECHANGE.currentZoneID][ix] == nil then
				print(string.format("New Node Location @ %d,%d", v.coordX, v.coordZ))
				ResourceTracker_Nodes[res][LIBZONECHANGE.currentZoneID][ix] = {x=v.coordX, z=v.coordZ, y=v.coordY, nodes = {} }
			end
			local lud = data.LOOKUP[v.description]
			ResourceTracker_Nodes[res][LIBZONECHANGE.currentZoneID][ix].nodes[lud.k] = true
		end
	end
end

local math_sqrt = math.sqrt
local math_atan = math.atan2
local math_abs = math.abs
local math_pi = math.pi

function RT.direction(dt)
	if dt > 180 then dt = dt-360 end
	if dt < -180 then dt = dt+360 end

	if dt >= 169		then	d = "S"
	elseif dt >= 146	then	d = "SSW"
	elseif dt >= 124	then	d = "SW"
	elseif dt >= 101	then	d = "WSW"
	elseif dt >= 79		then	d = "W"
	elseif dt >= 56		then	d = "WNW"
	elseif dt >= 34		then	d = "NW"
	elseif dt >= 11		then	d = "NNW"
	elseif dt >= -11	then	d = "N"
	elseif dt >= -34	then	d = "NNE"
	elseif dt >= -56	then	d = "NE"
	elseif dt >= -79	then	d = "ENE"
	elseif dt >= -101	then	d = "E"
	elseif dt >= -124	then	d = "ESE"
	elseif dt >= -146	then	d = "SE"
	elseif dt >= -169	then	d = "SSE"
	else						d = "S"
	end

	return {i="", l=d}
end

local CX_IMAGES = {
	["E"] = string.format("img/compass-E.png"),
	["ENE"] = string.format("img/compass-ENE.png"),
	["ESE"] = string.format("img/compass-ESE.png"),
	["N"] = string.format("img/compass-N.png"),
	["NE"] = string.format("img/compass-NE.png"),
	["NNE"] = string.format("img/compass-NNE.png"),
	["NNW"] = string.format("img/compass-NNW.png"),
	["NW"] = string.format("img/compass-NW.png"),
	["S"] = string.format("img/compass-S.png"),
	["SE"] = string.format("img/compass-SE.png"),
	["SSE"] = string.format("img/compass-SSE.png"),
	["SSW"] = string.format("img/compass-SSW.png"),
	["SW"] = string.format("img/compass-SW.png"),
	["W"] = string.format("img/compass-W.png"),
	["WNW"] = string.format("img/compass-WNW.png"),
	["WSW"] = string.format("img/compass-WSW.png"),

	["eE"] = string.format("img/Ecompass-E.png"),
	["eENE"] = string.format("img/Ecompass-ENE.png"),
	["eESE"] = string.format("img/Ecompass-ESE.png"),
	["eN"] = string.format("img/Ecompass-N.png"),
	["eNE"] = string.format("img/Ecompass-NE.png"),
	["eNNE"] = string.format("img/Ecompass-NNE.png"),
	["eNNW"] = string.format("img/Ecompass-NNW.png"),
	["eNW"] = string.format("img/Ecompass-NW.png"),
	["eS"] = string.format("img/Ecompass-S.png"),
	["eSE"] = string.format("img/Ecompass-SE.png"),
	["eSSE"] = string.format("img/Ecompass-SSE.png"),
	["eSSW"] = string.format("img/Ecompass-SSW.png"),
	["eSW"] = string.format("img/Ecompass-SW.png"),
	["eW"] = string.format("img/Ecompass-W.png"),
	["eWNW"] = string.format("img/Ecompass-WNW.png"),
	["eWSW"] = string.format("img/Ecompass-WSW.png")
}


local resort_req = false

function RT.ScanCurrentMap()
	if SVLOADED then
		for k,v in pairs(MESSAGE_STACK) do
			RT.UI.msgframes[k]:SetVisible(false)
		end
		MESSAGE_STACK = {}
		for k,v in pairs(Inspect.Map.Detail(Inspect.Map.List())) do
			if ResourceTracker_Settings.tracked[v.description] then
				RT.ShowMessage(v)
			end
		end
	end
end

local alpha=0

local rpt = 0

local last_update = 0

function RT.Event_System_Update_Begin(h)
	if #MESSAGE_STACK > 0 then
		local timenow = Inspect.Time.Real()
		if timenow - last_update > 0.1 then
			last_update = timenow
			local pd = Inspect.Unit.Detail("player")

			for x, v in pairs(MESSAGE_STACK) do
				if pd and pd.coordX and pd.coordZ then
					local dx = pd.coordX - MESSAGE_STACK[x].x
					local dy = pd.coordY - MESSAGE_STACK[x].y
					local dz = pd.coordZ - MESSAGE_STACK[x].z
					local d = math_sqrt((dx^2)+(dy^2)+(dz^2))
					local dr = math_atan(dx,dz)*180/math_pi
					local c = RT.direction(dr).l
					if math.abs(pd.coordY - MESSAGE_STACK[x].y) > 10 then
						MESSAGE_STACK[x].ixn = "e"..c
					else
						MESSAGE_STACK[x].ixn = c
					end
					MESSAGE_STACK[x].d=d
				else
					MESSAGE_STACK[x].d=1000
				end
			end

			table.sort(MESSAGE_STACK, function(a,b) return a.d < b.d end)

			local t = Inspect.Time.Real()
			local tf
			for x=ResourceTracker_Settings.num_nodes, 1, -1 do
				if MESSAGE_STACK[x] then
					if MESSAGE_STACK[x].t ~= nil then
						tf = t-MESSAGE_STACK[x].t
					else
						tf = 0
					end
					if tf > 2 then
						table.remove(MESSAGE_STACK, x)
						RT.UI.msgframes[x]:SetVisible(false)
						resort_req = true
					else
						if tf > 0 then
							alpha = (2-tf)/2
							RT.UI.msgframes[x]:SetAlpha(alpha)
							RT.UI.icons[x]:SetAlpha(alpha)
							RT.UI.dir[x]:SetAlpha(alpha)
						else
							RT.UI.msgframes[x]:SetAlpha(1)
							RT.UI.icons[x]:SetAlpha(1)
							RT.UI.dir[x]:SetAlpha(1)
						end
						if RT.UI.icons[x].ci ~= MESSAGE_STACK[x].n then
							RT.UI.icons[x]:SetTexture("Rift", ICONS[MESSAGE_STACK[x].n])
							RT.UI.icons[x].ci = MESSAGE_STACK[x].n
						end


						if RT.UI.dir[x].ci ~= MESSAGE_STACK[x].ixn and CX_IMAGES[MESSAGE_STACK[x].ixn] then
							RT.UI.dir[x]:SetTexture(addon.identifier, CX_IMAGES[MESSAGE_STACK[x].ixn])
							RT.UI.dir[x].ci = MESSAGE_STACK[x].ixn
						end

						if MESSAGE_STACK[x].d < 1000 then
							RT.UI.msgframes[x]:SetText(string.format("%s [%dm]", MESSAGE_STACK[x].n, MESSAGE_STACK[x].d))
						else
							RT.UI.msgframes[x]:SetText(string.format("%s", MESSAGE_STACK[x].n))
						end
						RT.UI.msgframes[x]:SetVisible(true)
					end
				else
					RT.UI.msgframes[x]:SetVisible(false)
				end
			end
			if resort_req == true then
				if #MESSAGE_STACK > 0 then
					local newdtls = {}
					local updt = false
					for x=1, ResourceTracker_Settings.num_nodes do
						if MESSAGE_STACK[x] then
							table.insert(newdtls, MESSAGE_STACK[x])
							updt = true
						end
					end
					if updt then
						MESSAGE_STACK = newdtls
					end
				end
				resort_req = false
			end
		end
	end
end

function RT.CreateConfig(k,v)
	local c = UI.CreateFrame("RiftCheckbox", "cb"..k, RT.UI.cfg_header)
	c:SetWidth(24)
	c:SetHeight(24)
	c:SetChecked(false)
	c:SetLayer(15)
	c:EventAttach(Event.UI.Checkbox.Change, function(self, h)
		if c:GetChecked() then
			ResourceTracker_Settings.tracked[v.name[LNG]] = true
		else
			ResourceTracker_Settings.tracked[v.name[LNG]] = nil
		end
		RT.ScanCurrentMap()
	end, "Event.UI.Checkbox.Change")
	local i = UI.CreateFrame("Texture", "i"..k, RT.UI.cfg_header)
	i:SetWidth(24)
	i:SetHeight(24)
	i:SetTexture("Rift", v.icon)
	i:SetPoint("TOPLEFT", c, "TOPRIGHT", 2,0)
	i:SetLayer(15)
	local t = UI.CreateFrame("Text", "t"..k, RT.UI.cfg_header)
	t:SetFontSize(12)
	t:SetText(v.name[LNG])
	t:SetPoint("CENTERLEFT", i, "CENTERRIGHT", 2,0)
	t:SetLayer(15)
	return c,i,t
end

function RT.BuildUI()
	RT.context = UI.CreateContext(addon.identifier)

	RT.UI = {
		msgframes = {}, icons = {}, dir = {}, configchk = {}
	}

	RT.UI.mm = UI.CreateFrame("Texture", "RT.UI.mm", RT.context)
	RT.UI.mm:SetHeight(36)
	RT.UI.mm:SetWidth(36)

	RT.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if ResourceTracker_Settings.locked or MINIMAPDOCKER ~= nil then
			RT.UI.cfg_header:SetVisible(not RT.UI.cfg_header:GetVisible())
			RT.UI.anchor:SetVisible(RT.UI.cfg_header:GetVisible())
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	RT.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if MINIMAPDOCKER == nil and ResourceTracker_Settings.locked == false then
			self.MouseDown = true
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - RT.UI.mm:GetLeft()
			self.sy = mouseData.y - RT.UI.mm:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Down")

	if MINIMAPDOCKER == nil then
		RT.UI.mm:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
			self.MouseDown = false
			if ResourceTracker_Settings.locked == false then
				ResourceTracker_Settings.mmx = RT.UI.mm:GetLeft()
				ResourceTracker_Settings.mmy = RT.UI.mm:GetTop()
			end
		end, "Event.UI.Input.Mouse.Left.Up")

		RT.UI.mm:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
			if ResourceTracker_Settings.locked == false then
				if self.MouseDown then
					local nx, ny
					local mouseData = Inspect.Mouse()
					nx = mouseData.x - self.sx
					ny = mouseData.y - self.sy
					RT.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
				end
			end
		end, "Event.UI.Input.Mouse.Cursor.Move")

		RT.UI.mm:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
			self.MouseDown = false
			ResourceTracker_Settings.locked = not ResourceTracker_Settings.locked
			if ResourceTracker_Settings.locked == false then
				RT.UI.mm:SetTexture(addon.identifier, "img/mm_button_bw.png")
			else
				RT.UI.mm:SetTexture(addon.identifier, "img/mm_button.png")
			end
		end, "Event.UI.Input.Mouse.Right.Click")
	end

	RT.UI.cfg_header = UI.CreateFrame("Text", "RT.UI.cfg_header", RT.context)
	RT.UI.cfg_header:SetVisible(false)
	RT.UI.cfg_header:SetBackgroundColor(0,0,1)
	RT.UI.cfg_header:SetFontSize(14)
	RT.UI.cfg_header:SetLayer(1)
	RT.UI.cfg_header:SetText("RT Config")

	RT.UI.close = UI.CreateFrame("RiftButton", "RT.UI.close", RT.UI.cfg_header)
	RT.UI.close:SetSkin("close")
	RT.UI.close:SetPoint("TOPRIGHT", RT.UI.cfg_header, "TOPRIGHT", 0,0)
	RT.UI.close:SetHeight(RT.UI.cfg_header:GetHeight())
	RT.UI.close:SetWidth(RT.UI.cfg_header:GetHeight())
	RT.UI.close:SetLayer(3)

	RT.UI.close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		RT.UI.cfg_header:SetVisible(false)
		RT.UI.anchor:SetVisible(false)
	end, "Event.UI.Input.Mouse.Left.Click")

	RT.UI.config = UI.CreateFrame("Frame", "RT.UI.config", RT.UI.cfg_header )
	RT.UI.config:SetPoint("TOPCENTER", RT.UI.cfg_header, "BOTTOMCENTER", 0,0)
	RT.UI.config:SetBackgroundColor(0,0,0)
	RT.UI.config:SetLayer(5)

	RT.UI.cfg_header:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - RT.UI.cfg_header:GetLeft()
		self.sy = mouseData.y - RT.UI.cfg_header:GetTop()
	end, "Event.UI.Input.Mouse.Left.Down")

	RT.UI.cfg_header:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		ResourceTracker_Settings.cfgx = RT.UI.cfg_header:GetLeft()
		ResourceTracker_Settings.cfgy = RT.UI.cfg_header:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")

	RT.UI.cfg_header:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			RT.UI.cfg_header:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	local frm_metal = nil
	local frm_wood = nil
	local frm_plants = nil
	local frm_fish = nil

	local frm_p = nil

	local txt_w = 0

	for k,v in pairs(ITEMS["MINING"]) do
		local c,i,t = RT.CreateConfig(k,v)
		RT.UI.configchk[v.name[LNG]] = {c=c, i=i, t=t}
		txt_w = math.max(txt_w, t:GetWidth())

		if frm_p == nil then
			frm_metal = c
		else
			c:SetPoint("TOPLEFT", frm_p, "BOTTOMLEFT", 0,0)
		end

		frm_p = c
		ICONS[v.name[LNG]] = v.icon
		RT.ALLITEMS[v.name[LNG]] = IDX_ORE
	end

	frm_p = nil
	for k,v in pairs(ITEMS["WOOD"]) do
		local c,i,t = RT.CreateConfig(k,v)
		RT.UI.configchk[v.name[LNG]] = {c=c, i=i, t=t}
		txt_w = math.max(txt_w, t:GetWidth())

		if frm_p == nil then
			frm_wood = c
		else
			c:SetPoint("TOPLEFT", frm_p, "BOTTOMLEFT", 0,0)
		end

		frm_p = c
		ICONS[v.name[LNG]] = v.icon
		RT.ALLITEMS[v.name[LNG]] = IDX_WOOD
	end

	frm_p = nil
	for k,v in pairs(ITEMS["PLANTS"]) do
		local c,i,t = RT.CreateConfig(k,v)
		RT.UI.configchk[v.name[LNG]] = {c=c, i=i, t=t}
		txt_w = math.max(txt_w, t:GetWidth())

		if frm_p == nil then
			frm_plants = c
		else
			c:SetPoint("TOPLEFT", frm_p, "BOTTOMLEFT", 0,0)
		end

		frm_p = c
		ICONS[v.name[LNG]] = v.icon
		RT.ALLITEMS[v.name[LNG]] = IDX_PLANTS
	end

	frm_p = nil
	for k,v in pairs(ITEMS["FISH"]) do
		local c,i,t = RT.CreateConfig(k,v)
		RT.UI.configchk[v.name[LNG]] = {c=c, i=i, t=t}
		txt_w = math.max(txt_w, t:GetWidth())

		if frm_p == nil then
			frm_fish = c
		else
			c:SetPoint("TOPLEFT", frm_p, "BOTTOMLEFT", 0,0)
		end

		frm_p = c
		ICONS[v.name[LNG]] = v.icon
		RT.ALLITEMS[v.name[LNG]] = IDX_FISH
	end

	for k,v in pairs(RT.UI.configchk) do
		v.t:SetWidth(txt_w)
	end

	frm_metal:SetPoint("TOPLEFT", RT.UI.cfg_header, "BOTTOMLEFT", 2,2)

	local colwidth = 52 + txt_w

	frm_wood:SetPoint("TOPLEFT", frm_metal, "TOPLEFT", colwidth+2, 0)
	frm_plants:SetPoint("TOPLEFT", frm_wood, "TOPLEFT", colwidth+2, 0)
	frm_fish:SetPoint("TOPLEFT", frm_plants, "TOPLEFT", colwidth+2, 0)

	RT.UI.config:SetHeight(800)

	local frmht = math.max(#ITEMS["MINING"], #ITEMS["WOOD"], #ITEMS["PLANTS"], #ITEMS["FISH"])

	RT.UI.config:SetWidth((colwidth*4) + 10)
	RT.UI.config:SetHeight((24 * frmht) + 4)
	RT.UI.cfg_header:SetWidth((colwidth*4) + 10)

	RT.UI.cfgoptions = UI.CreateFrame("Frame", "RT.UI.cfgoptions", RT.UI.config)
	RT.UI.cfgoptions:SetWidth((colwidth*4) + 10)
	RT.UI.cfgoptions:SetHeight(60)
	RT.UI.cfgoptions:SetLayer(1)
	RT.UI.cfgoptions:SetBackgroundColor(0,0,0,1)
	RT.UI.cfgoptions:SetPoint("TOPLEFT", RT.UI.config, "BOTTOMLEFT", 0,0)

	RT.UI.l_nodes = UI.CreateFrame("Text", "RT.UI.l_nodes", RT.UI.cfgoptions)
	RT.UI.l_nodes:SetPoint("TOPLEFT", RT.UI.cfgoptions, "TOPLEFT", 2,2)
	RT.UI.l_nodes:SetFontSize(14)
	RT.UI.l_nodes:SetLayer(5)
	RT.UI.l_nodes:SetText("Number of nodes to display: ")

	RT.UI.s_nodes = UI.CreateFrame("RiftSlider", "RT.UI.s_nodes", RT.UI.cfgoptions)
	RT.UI.s_nodes:SetLayer(2)
	RT.UI.s_nodes:SetWidth(math.floor(colwidth*1.5))
	RT.UI.s_nodes:SetRange(1,10)
	RT.UI.s_nodes:SetPoint("TOPLEFT", RT.UI.l_nodes, "BOTTOMLEFT", 16,0)

	RT.UI.s_nodes:EventAttach(Event.UI.Slider.Change, function(self, h)
		ResourceTracker_Settings.num_nodes = RT.UI.s_nodes:GetPosition()
		RT.UI.l_nodes:SetText(string.format("Number of nodes to display: %d", ResourceTracker_Settings.num_nodes))
		RT.ScanCurrentMap()
	end, "Event.UI.Slider.Change")

	RT.UI.l_duration = UI.CreateFrame("Text", "RT.UI.l_duration", RT.UI.cfgoptions)
	RT.UI.l_duration:SetPoint("TOPLEFT", RT.UI.l_nodes, "TOPLEFT", (colwidth*2)+4,2)
	RT.UI.l_duration:SetFontSize(14)
	RT.UI.l_duration:SetLayer(5)
	RT.UI.l_duration:SetText("Seconds to keep onscreen: ")

	RT.UI.s_duration = UI.CreateFrame("RiftSlider", "RT.UI.s_duration", RT.UI.cfgoptions)
	RT.UI.s_duration:SetLayer(2)
	RT.UI.s_duration:SetWidth(math.floor(colwidth*1.5))
	RT.UI.s_duration:SetRange(0,10)
	RT.UI.s_duration:SetPoint("TOPLEFT", RT.UI.l_duration, "BOTTOMLEFT", 16,0)
	RT.UI.s_duration:EventAttach(Event.UI.Slider.Change, function(self, h)
		ResourceTracker_Settings.fade_time = RT.UI.s_duration:GetPosition()
		RT.UI.l_duration:SetText(string.format("Seconds to keep onscreen: %d", ResourceTracker_Settings.fade_time))
	end, "Event.UI.Slider.Change")

	RT.UI.c_combat = UI.CreateFrame("RiftCheckbox", "RT.UI.c_combat", RT.UI.cfgoptions)
	RT.UI.c_combat:SetWidth(24)
	RT.UI.c_combat:SetHeight(24)
	RT.UI.c_combat:SetLayer(5)
	RT.UI.c_combat:SetPoint("TOPLEFT", RT.UI.l_nodes, "BOTTOMLEFT", 0, 14)
	RT.UI.c_combat:EventAttach(Event.UI.Checkbox.Change, function(self, h)
		if RT.UI.c_combat:GetChecked() then
			ResourceTracker_Settings.showcombat = true
		else
			ResourceTracker_Settings.showcombat = false
		end
	end, "Event.UI.Checkbox.Change")

	RT.UI.l_combat = UI.CreateFrame("Text", "RT.UI.l_combat", RT.UI.cfgoptions)
	RT.UI.l_combat:SetFontSize(14)
	RT.UI.l_combat:SetLayer(15)
	RT.UI.l_combat:SetText("Show in combat")
	RT.UI.l_combat:SetPoint("CENTERLEFT", RT.UI.c_combat, "CENTERRIGHT", 2,0)

	RT.UI.configscale = UI.CreateFrame("Frame", "RT.UI.configscale", RT.UI.config)
	RT.UI.configscale:SetWidth((colwidth*4) + 10)
	RT.UI.configscale:SetHeight(48)
	RT.UI.configscale:SetLayer(1)
	RT.UI.configscale:SetBackgroundColor(0.25,0,0,1)
	RT.UI.configscale:SetPoint("TOPLEFT", RT.UI.cfgoptions, "BOTTOMLEFT", 0,0)

	RT.UI.configscale_ctr = UI.CreateFrame("Frame", "RT.UI.configscale_ctr", RT.UI.config)
	RT.UI.configscale_ctr:SetPoint("CENTER", RT.UI.configscale, "CENTER", 0,0)
	RT.UI.configscale_ctr:SetLayer(2)

	RT.UI.configscale:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if ResourceTracker_Settings.fontsize <= 128 then
			ResourceTracker_Settings.fontsize = ResourceTracker_Settings.fontsize+1
			RT.ResizeElements()
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	RT.UI.configscale:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if ResourceTracker_Settings.fontsize >= 12 then
			ResourceTracker_Settings.fontsize = ResourceTracker_Settings.fontsize-1
			RT.ResizeElements()
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")

	RT.UI.cs_d = UI.CreateFrame("Texture", "RT.UI.cs_d", RT.UI.config)
	RT.UI.cs_d:SetPoint("TOPLEFT", RT.UI.configscale_ctr, "TOPLEFT", 0,0)
	RT.UI.cs_d:SetTexture(addon.identifier, "img/compass-N.png")
	RT.UI.cs_d:SetLayer(5)

	RT.UI.cs_i = UI.CreateFrame("Texture", "RT.UI.cs_i", RT.UI.config)
	RT.UI.cs_i:SetPoint("TOPLEFT", RT.UI.cs_d, "TOPRIGHT", 0,0)
	RT.UI.cs_i:SetTexture("Rift", "Data\\UI\\item_icons\\ore02.dds")
	RT.UI.cs_i:SetLayer(5)

	RT.UI.cs_t = UI.CreateFrame("Text", "RT.UI.cs_t", RT.UI.config)
	RT.UI.cs_t:SetPoint("CENTERLEFT", RT.UI.cs_i, "CENTERRIGHT", 0,0)
	RT.UI.cs_t:SetText("Resouce Node [123m]")
	RT.UI.cs_t:SetLayer(5)

	local fht = 0

	--local bgcols = { {0,0,0}, {0,0,1}, {0,1,0}, {0,1,1}, {1,0,0}, {1,0,1}, {0.8,0.8,0.8} }

	for x=1,10 do
		RT.UI.msgframes[x] = UI.CreateFrame("Text", "RT.msgframe", RT.context)
		RT.UI.msgframes[x]:SetFontColor(1,1,1,1)
		RT.UI.icons[x] = UI.CreateFrame("Texture", "RT.icon", RT.UI.msgframes[x])
		RT.UI.icons[x]:SetPoint("TOPRIGHT", RT.UI.msgframes[x], "TOPLEFT")
		RT.UI.icons[x].ci = ""
		RT.UI.dir[x] = UI.CreateFrame("Texture", "RT.icon", RT.UI.msgframes[x])
		RT.UI.dir[x]:SetPoint("TOPRIGHT", RT.UI.icons[x], "TOPLEFT")
		RT.UI.dir[x].ci = ""

		if x > 1 then
			RT.UI.msgframes[x]:SetPoint("TOPLEFT", RT.UI.msgframes[x-1], "BOTTOMLEFT", 0,0)
		end
--		if x <= 7 then
--			RT.UI.msgframes[x]:SetBackgroundColor(bgcols[x][1], bgcols[x][2], bgcols[x][3])
--			RT.UI.icons[x]:SetBackgroundColor(bgcols[x][1], bgcols[x][2], bgcols[x][3])
--			RT.UI.dir[x]:SetBackgroundColor(bgcols[x][1], bgcols[x][2], bgcols[x][3])
--		end
	end

	RT.UI.FRAMESIZE = UI.CreateFrame("Text", "RT.UI.FRAMESIZE", RT.context)
	RT.UI.FRAMESIZE:SetVisible(false)
	RT.UI.FRAMESIZE:SetText("QQqq")
	RT.UI.FRAMESIZE:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -256, -256)

	RT.UI.anchor = UI.CreateFrame("Text", "RT.UI.anchor", RT.context)
	RT.UI.anchor:SetVisible(false)
	RT.UI.anchor:SetFontSize(14)
	RT.UI.anchor:SetText("Resource Tracker")
	RT.UI.anchor:SetBackgroundColor(0,0,1,1)
	RT.UI.anchor:SetFontColor(1,1,1,1)

	RT.UI.anchor:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		self.MouseDown = true
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - RT.UI.msgframes[1]:GetLeft()
		self.sy = mouseData.y - RT.UI.msgframes[1]:GetTop()
	end, "Event.UI.Input.Mouse.Left.Down")

	RT.UI.anchor:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			RT.UI.msgframes[1]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	RT.UI.anchor:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		if self.MouseDown then
			self.MouseDown = false
		end
		ResourceTracker_Settings.trx = RT.UI.msgframes[1]:GetLeft()
		ResourceTracker_Settings.try = RT.UI.msgframes[1]:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")
end

function RT.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if ResourceTracker_Settings == nil then ResourceTracker_Settings = {} end
		if ResourceTracker_Nodes == nil then ResourceTracker_Nodes = {} end
		
		if ResourceTracker_Nodes.version == nil or ResourceTracker_Nodes.version < default_settings.version then
print("NEED TO UPDATE SAVED DATA")		
			if ResourceTracker_Nodes.version == nil then
print("No version info. So using original saved variables.")
				local cx
				local cidx
				local newRTS = {}
				local lud
				local errors = {}
				for tk, tv in pairs(ResourceTracker_Nodes) do
					newRTS[tk] = {}
					for zk,zv in pairs(tv) do
						newRTS[tk][zk] = {}
						for ik,iv in pairs(zv) do
							cx = {0,0,0}
							cidx = 0
							for tkn in string.gmatch(ik, "[0-9]+") do cidx=cidx+1 cx[cidx]=tkn end
							newRTS[tk][zk][ik] = { x=cx[1], z=cx[2], y=cx[3], nodes = {} }
							for rk, rv in pairs(iv) do
								lud = data.LOOKUP[rk]
								if lud then
									newRTS[tk][zk][ik].nodes[lud.k] = true
								else
									if errors[rk] == nil then
										print("Unable to lookup: "..rk)
										errors[rk] = true
									end
								end
							end
						end
					end
				end
				
				ResourceTracker_Nodes = {}
				MergeTable(ResourceTracker_Nodes, newRTS)
			else
			end
		end
		
		MergeTable(ResourceTracker_Settings, default_settings)
		MergeTable(ResourceTracker_Nodes, default_settings_node)
		
--		ResourceTracker_Settings.version = nil
		
		if MINIMAPDOCKER then
			MINIMAPDOCKER.Register(addon.identifier, RT.UI.mm)
			RT.UI.mm:SetTexture(addon.identifier, "img/mm_button.png")
		else
			RT.UI.mm:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ResourceTracker_Settings.mmx, ResourceTracker_Settings.mmy)
			if ResourceTracker_Settings.locked then
				RT.UI.mm:SetTexture(addon.identifier, "img/mm_button.png")
			else
				RT.UI.mm:SetTexture(addon.identifier, "img/mm_button_bw.png")
			end
		end

		RT.ResizeElements()

		if ResourceTracker_Settings.trx == -1 then
			RT.UI.msgframes[1]:SetPoint("TOPLEFT", UIParent, "CENTER", 0,-100)
			ResourceTracker_Settings.trx = RT.UI.msgframes[1]:GetLeft()
			ResourceTracker_Settings.try = RT.UI.msgframes[1]:GetTop()
		else
			RT.UI.msgframes[1]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ResourceTracker_Settings.trx, ResourceTracker_Settings.try)
		end

		for k,v in pairs(ResourceTracker_Settings.tracked) do
			if RT.UI.configchk[k] then
				RT.UI.configchk[k].c:SetChecked(true)
			end
		end
		RT.UI.cfg_header:SetPoint("TOPLEFT", UIParent, "TOPLEFT", ResourceTracker_Settings.cfgx, ResourceTracker_Settings.cfgy)
		local nn = ResourceTracker_Settings.num_nodes
		RT.UI.s_nodes:SetPosition(2)
		RT.UI.s_nodes:SetPosition(nn)
		local ft = ResourceTracker_Settings.fade_time
		RT.UI.s_duration:SetPosition(2)
		RT.UI.s_duration:SetPosition(ft)
		RT.UI.c_combat:SetChecked(ResourceTracker_Settings.showcombat)
		SVLOADED = true
		RT.ScanCurrentMap()
	end
end

function RT.ResizeElements()
	RT.UI.FRAMESIZE:SetFontSize(ResourceTracker_Settings.fontsize)
	RT.UI.fht = RT.UI.FRAMESIZE:GetHeight()

	for x=1, 10 do
		RT.UI.msgframes[x]:SetFontSize(ResourceTracker_Settings.fontsize)
		RT.UI.icons[x]:SetWidth(RT.UI.fht)
		RT.UI.icons[x]:SetHeight(RT.UI.fht)
		RT.UI.dir[x]:SetWidth(RT.UI.fht)
		RT.UI.dir[x]:SetHeight(RT.UI.fht)
	end

	RT.UI.anchor:SetPoint("BOTTOMLEFT", RT.UI.msgframes[1], "TOPLEFT", -(RT.UI.fht*2), 0)
	RT.UI.anchor:SetWidth(RT.UI.fht * 8)

	RT.UI.cs_d:SetWidth(RT.UI.fht)
	RT.UI.cs_d:SetHeight(RT.UI.fht)
	RT.UI.cs_i:SetWidth(RT.UI.fht)
	RT.UI.cs_i:SetHeight(RT.UI.fht)
	RT.UI.cs_t:SetFontSize(ResourceTracker_Settings.fontsize)
	RT.UI.configscale_ctr:SetHeight(RT.UI.fht)
	RT.UI.configscale_ctr:SetWidth(RT.UI.fht*10)
end

function RT.Event_System_Secure_Enter(h)
	if ResourceTracker_Settings.showcombat == false then
		RT_ACTIVE = false
		for k,v in pairs(MESSAGE_STACK) do
			MESSAGE_STACK[k].t = 0
		end
	end
end

function RT.Event_System_Secure_Leave(h)
	if ResourceTracker_Settings.showcombat == false then
		RT_ACTIVE = true
		RT.ScanCurrentMap()
	end
end

RT.BuildUI()

Command.Event.Attach(Event.Map.Add, RT.Event_Map_Add, "Event.Map.Add")
Command.Event.Attach(Event.Map.Remove, RT.Event_Map_Remove, "Event.Map.Remove")
Command.Event.Attach(Event.System.Update.Begin, RT.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, RT.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")
Command.Event.Attach(Event.System.Secure.Leave, RT.Event_System_Secure_Leave, "Event.System.Secure.Leave")
Command.Event.Attach(Event.System.Secure.Enter, RT.Event_System_Secure_Enter, "Event.System.Secure.Enter")

print(string.format("v%s loaded.", addon.toc.Version))
