--[[
File: Interface.lua
Author: Zormus
--]]

local Interface = Library.LibZormus.Interface
local Utility = Library.LibZormus.Utility
local _, Roles = ...

Roles.minWidth = 100
Roles.maxWidth = 260
Roles.defaultWidth = 160
Roles.menuSpacing = 2

local context = Interface.Create("Context", "Menu")
context:SetSecureMode("restricted")
Roles.menu = Interface.Create(Roles.Interface.Menu, "Button", context)
local menu = Roles.menu

local function InitializeRoleDisplay(self)
	self.iconFrame = Interface.Create("Image", "IconFrame", self)
	self.iconFrame:SetImage("sml_icon_border.png.dds")
	self.iconFrame:SetSize(26, 26)
	self.iconFrame:SetPoint(0, 0.5, self, 0, 0.5, 5, 0)
	self.icon = Interface.Create("Image", "Icon", self)
	self.icon:SetPoint("CENTER", self.iconFrame, "CENTER")
	self.icon:SetLayer(self.iconFrame:GetLayer() - 1)
	self.label = Interface.Create("Text", "Label", self)
	self.label:SetPoint("LEFT", self.iconFrame, "RIGHT", 4, nil)
	self.label:SetPoint(1, 0.5, self, 1, 0.5, -4, 0)
	self.label:SetFontColor(0.96, 0.87, 0.74, 1)
	self.label:SetFontSize(15)
	self.label:SetEffectGlow({colorR = 0.54, colorG = 0.42, colorB = 0.22, strength = 1.5})
end

local function OnHighlight(self, highlight)
	if highlight then
		self.iconFrame:SetImage("sml_icon_border_(over)_blue.png.dds")
	else
		self.iconFrame:SetImage("sml_icon_border.png.dds")
	end
end

local function InitializeButton(button)
	button:SetHeight(40)
	button:SetBevel(3, 4)
	button:SetColor(0, 0, 0, 0.5)
	button:SetHighlightColor(0, 0, 0, 0.3)
	button:SetShadowColor(0, 0, 0, 0.7)
	InitializeRoleDisplay(button:GetFace())
	function button.OnHighlight(self, highlight)
		OnHighlight(self:GetFace(), highlight)
	end
	button:EnableHighlight(true)
end

function InitializeList(list)
	list:SetItemHeight(36)
	list:SetBackgroundColor(0, 0, 0, 0.5)
	function list.OnItemHighlight(item, highlight)
		if (highlight) then
			item:SetBackgroundColor(0.7, 0.7, 0.7, 0.1)
		else
			item:SetBackgroundColor(0, 0, 0, 0)
		end
		OnHighlight(item, highlight)
	end
	function list.InitializeItem(item)
		InitializeRoleDisplay(item)
	end
end

InitializeButton(menu)
InitializeList(menu:GetList())
Command.Event.Attach(Event.System.Secure.Enter, function() menu:SetActive(false) end, "SystemSecureEnter")
Command.Event.Attach(Event.System.Secure.Leave, function() if not Roles.config:IsOpen() then menu:SetActive(true) end end, "SystemSecureLeave")

local icons =
{
	tank = "role_tank_big.png.dds",
	heal = "role_heal_big.png.dds",
	dps = "role_damage_big.png.dds",
	support = "role_support_big.png.dds"
}

local color =
{
	tank = "3366FF",
	heal = "33CC33",
	dps = "FF3333",
	support = "CC33FF",
	none = "999999"
}

local function UpdateRoleDisplay(self, role)
	self.icon:SetImage(icons[role:GetType()])
	self.label:SetText(role:GetName())
end

function menu:UpdateItem(index, role)
	local item = self:GetList():GetItem(index)
	UpdateRoleDisplay(item, role)
	item.color = color[role:GetType()]
	for _, button in ipairs({"Left", "Right"}) do
		item:EventMacroSet(Event.UI.Input.Mouse[button].Click, role:GetMacro(button))
	end
end

function menu:UpdateButton(role)
	UpdateRoleDisplay(self:GetFace(), role)
end

function menu.OnItemClick(item, button)
	local message = string.format("Changing role: <font color=\"#%s\">%s</font>", item.color, item.label:GetText())
	Utility.Message("system", true, message, true)
end
