--[[
File: Interface/Menu.lua
Author: Zormus
--]]

local Interface = Library.LibZormus.Interface
local _, Roles = ...
Roles.Interface = Roles.Interface or {}
Roles.Interface.Menu = setmetatable({}, Roles.Interface.Button)
local Menu = Roles.Interface.Menu
Menu.__index = Menu

function Menu:Initialize()
	Roles.Interface.Button.Initialize(self)
	self.OnLeftClick = Menu.Toggle
	self:EnableClick(true, "Left")
	self.list = Interface.Create(Roles.Interface.List, "List", self:GetParent())
	self.list:SetSecureMode("restricted")
	self.list:SetVisible(false)
	function self.list.OnItemClick(item, button)
		self:Toggle()
		if self.OnItemClick then
			self.OnItemClick(item, button)
		end
	end
end

function Menu:GetList()
	return self.list
end

function Menu:SetLayout(extend, spacing)
	if extend == "Down" then
		self.list:ClearPoint("BOTTOMLEFT")
		self.list:ClearPoint("BOTTOMRIGHT")
		self.list:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, spacing)
		self.list:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, spacing)
	elseif extend == "Up" then
		self.list:ClearPoint("TOPLEFT")
		self.list:ClearPoint("TOPRIGHT")
		self.list:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, -spacing)
		self.list:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", 0, -spacing)
	end
	self.extend = extend
end

function Menu:SetActive(active)
	self:EnableClick(active, "Left")
	if active then
		self:SetAlpha(1)
	else
		if self:IsOpen() then
			self:Toggle()
		end
		self:SetAlpha(0.5)
	end
end

function Menu:IsOpen()
	return self:GetPressed()
end

function Menu:Toggle()
	if not self:IsOpen() then
		if self.extend ~= _config.extend then
			self:SetLayout(_config.extend, Roles.menuSpacing)
		end
		if self.OnOpen then
			self:OnOpen()
		end
	end
	Roles.Interface.Button.Toggle(self)
	self.list:SetVisible(self:IsOpen())
end

function Menu:SetCount(count)
	self.list:SetCount(count)
end
